library(testthat)
context("Seasonal daily data")

source("modules/Loading/Loading.R")
source("modules/Skill/Skill.R")
source("modules/Downscaling/Downscaling.R")


recipe_file <- "tests/recipes/recipe-seasonal_downscaling.yml"
recipe <- prepare_outputs(recipe_file, disable_checks = F)

# Load datasets
suppressWarnings({invisible(capture.output(
data <- Loading(recipe)
))})

# Downscale the data
suppressWarnings({invisible(capture.output(
downscaled_data <- Downscaling(recipe, data)
))})

# Compute skill metrics
suppressWarnings({invisible(capture.output(
skill_metrics <- Skill(recipe, downscaled_data)
))})

test_that("1. Loading", {

expect_equal(
is.list(data),
TRUE
)
expect_equal(
names(data),
c("hcst", "fcst", "obs")
)
expect_equal(
class(data$hcst),
"s2dv_cube"
)
expect_equal(
data$fcst,
NULL
)
expect_equal(
class(data$obs),
"s2dv_cube"
)
expect_equal(
names(data$hcst),
c("data", "dims", "coords", "attrs")
)
expect_equal(
names(data$hcst),
names(data$obs)
)
expect_equal(
dim(data$hcst$data),
c(dat = 1, var = 1, sday = 1, sweek = 1, syear = 4, time = 31, latitude = 4, longitude = 4, ensemble = 25)
)
expect_equal(
dim(data$obs$data),
c(dat = 1, var = 1, sday = 1, sweek = 1, syear = 4, time = 31, latitude = 13, longitude = 13, ensemble = 1)
)
expect_equal(
dim(data$obs$attrs$Dates),
c(sday = 1, sweek = 1, syear = 4, time = 31)
)
expect_equal(
as.vector(drop(data$hcst$data)[1:2,1:2,1,2,3]),
c(295.5691, 291.7752, 294.0874, 290.1173),
tolerance = 0.0001
)
expect_equal(
mean(data$hcst$data),
288.3723,
tolerance = 0.0001
)
expect_equal(
range(data$hcst$data),
c(280.1490, 298.2324),
tolerance = 0.0001
)
expect_equal(
(data$hcst$attrs$Dates)[1],
as.POSIXct("1993-12-01 18:00:00 UTC", tz = 'UTC')
)
expect_equal(
(data$hcst$attrs$Dates)[2],
as.POSIXct("1994-12-01 18:00:00 UTC", tz = 'UTC')
)
expect_equal(
(data$hcst$attrs$Dates)[5],
as.POSIXct("1993-12-02 18:00:00 UTC", tz = 'UTC')
)
expect_equal(
(data$obs$attrs$Dates)[10],
as.POSIXct("1994-12-03 11:30:00 UTC", tz = 'UTC')
)

})

#======================================
test_that("2. Downscaling", {

expect_equal(
is.list(downscaled_data),
TRUE
)
expect_equal(
names(downscaled_data),
c("hcst", "obs", "fcst")
)
expect_equal(
class(downscaled_data$hcst),
"s2dv_cube"
)
expect_equal(
class(downscaled_data$obs),
"s2dv_cube"
)
expect_equal(
downscaled_data$fcst,
NULL
)
expect_equal(
names(downscaled_data$hcst),
c("data", "dims", "coords", "attrs")
)
expect_equal(
names(downscaled_data$hcst),
names(downscaled_data$obs)
)
expect_equal(
dim(downscaled_data$hcst$data),
c(latitude = 13, longitude = 13, syear = 4, dat = 1, var = 1, sday = 1, sweek = 1, time = 31, ensemble = 25)
)
expect_equal(
dim(downscaled_data$obs$data),
c(ensemble = 1, dat = 1, var = 1, sday = 1, sweek = 1, syear = 4, time = 31, latitude = 13, longitude = 13)
)
expect_equal(
as.vector(drop(downscaled_data$hcst$data)[1:2,1:2,1,2,3]),
c(293.8468, 293.5830, 294.0713, 293.7104),
tolerance = 0.0001
)
expect_equal(
mean(downscaled_data$hcst$data),
288.5359,
tolerance = 0.0001
)
expect_equal(
range(downscaled_data$hcst$data),
c(284.0950, 297.4405),
tolerance = 0.0001
)

})

#======================================
test_that("3. Metrics", {

expect_equal(
is.list(skill_metrics),
TRUE
)
expect_equal(
names(skill_metrics),
c("bss10", "bss10_significance", "crpss", "crpss_significance","rpss", "rpss_significance", "mean_bias")
)
expect_equal(
class(skill_metrics$rpss),
"array"
)
expect_equal(
dim(skill_metrics$rpss),
c(latitude = 13, longitude = 13, var = 1, time = 31)
)
expect_equal(
dim(skill_metrics$rpss_significance),
dim(skill_metrics$rpss)
)
expect_equal(
skill_metrics$rpss[1],
-0.05325714,
tolerance = 0.0001
)
expect_equal(
skill_metrics$rpss_significance[1],
FALSE
)

})

unlink(recipe$Run$output_dir, recursive = TRUE)
